/* Author: Liuyuqaing */
	      
#include <iostream>      
#include <unistd.h>
#include <stdlib.h>
#include <libgen.h>
#include <fstream>

#include <ros/ros.h>

#include <tf/transform_listener.h>
#include <tf/transform_broadcaster.h>
#include <geometry_msgs/Pose.h>

#include <nav_msgs/Path.h>

#include "yaml-cpp/yaml.h"

 #ifdef HAVE_YAMLCPP_GT_0_5_0
// The >> operator disappeared in yaml-cpp 0.5, so this function is
// added to provide support for code written under the yaml-cpp 0.3 API.
template<typename T>
void operator >> (const YAML::Node& node, T& i)
{
	i = node.as<T>();
}
#endif

class getPath
{
public:
	getPath()
	{
		char pPath[256] = {0};
		int ret_ = -1;
		std::string path_str;
				
		ret_ = readlink("/proc/self/exe", pPath, 256);
		pPath[ret_] = '\0';

		path_str = pPath;
		path_str = path_str.substr(0, path_str.length() - 16);
		path_str += "param/params.yaml";
		//std::cout<<"测试path："<<(path_str)<<std::endl;
		
		std::ifstream fin(path_str.c_str());
		if (fin.fail()) 
		{
			ROS_ERROR("Map_server could not open %s.", path_str.c_str());
			exit(-1);
		}
			
#ifdef HAVE_YAMLCPP_GT_0_5_0
        // The document loading process changed in yaml-cpp 0.5.
        YAML::Node doc = YAML::Load(fin);
#else
        YAML::Parser parser(fin);
        YAML::Node doc;
        parser.GetNextDocument(doc);
#endif		//解析yaml文件
		try {
				doc["sampling_fre"] >> sampling_fre;
		} catch (YAML::InvalidScalar) {
			ROS_ERROR("The map does not contain a sampling_fre tag or it is invalid.");
			exit(-1);
		}
		try {
				doc["sampling_dist"] >> sampling_dist;
		} catch (YAML::InvalidScalar) {
			ROS_ERROR("The map does not contain a sampling_dist tag or it is invalid.");
			exit(-1);
		}
		try {
				doc["robot_link"] >> robot_link;
		} catch (YAML::InvalidScalar) {
			ROS_ERROR("The map does not contain a robot_link tag or it is invalid.");
			exit(-1);
		}
		try {
				doc["map_link"] >> map_link;
		} catch (YAML::InvalidScalar) {
			ROS_ERROR("The map does not contain a map_link tag or it is invalid.");
			exit(-1);
		}
		try {
				doc["pose_topic"] >> pose_topic;
		} catch (YAML::InvalidScalar) {
			ROS_ERROR("The map does not contain a pose_topic tag or it is invalid.");
			exit(-1);
		}
		
		ros::NodeHandle path_h;
		path_pub_ = path_h.advertise<nav_msgs::Path>(pose_topic, 1, true);
	
		bool got_tf = false;
		tf::StampedTransform world_pose;
		tf::TransformListener listener_;
		geometry_msgs::Pose pose_last, pose_now;
		geometry_msgs::PoseStamped path_stamped;
		
		robot_trajectory.header.frame_id=map_link;
		path_stamped.header.frame_id=map_link;
		pose_last.position.x = 0;
		pose_last.position.y = 0;
		
		ros::Rate rate_(sampling_fre);
		while(ros::ok())
		{
			// Update the pose of the robot
			got_tf = false;
			while(!got_tf){
				try{
					listener_.lookupTransform(map_link, robot_link, ros::Time(0), world_pose);
					got_tf = true;
				}
					catch (tf::TransformException ex) {
					ROS_INFO_STREAM_THROTTLE( 20,"Wait for tf: from /map to /base_link !"); 
				} 
				ros::Duration(0.02).sleep();
			}
			if(got_tf)
			{
				pose_now.position.x = world_pose.getOrigin().getX();
				pose_now.position.y = world_pose.getOrigin().getY();
				pose_now.position.z = world_pose.getOrigin().getZ();
				pose_now.orientation.x = world_pose.getRotation().getX();
				pose_now.orientation.y = world_pose.getRotation().getY();
				pose_now.orientation.z = world_pose.getRotation().getZ();
				pose_now.orientation.w = world_pose.getRotation().getW();

				bool update_flag = false;
				if( (pose_last.position.x == 0) && (pose_last.position.y == 0) )
				{
					pose_last = pose_now;
					update_flag = true;
				}else{
					double dist = sqrt(pow(pose_now.position.x - pose_last.position.x, 2) + 
						pow(pose_now.position.y - pose_last.position.y, 2) + 
						pow(pose_now.position.z - pose_last.position.z, 2));
					
					if(dist > sampling_dist)
					{
						pose_last = pose_now;
						update_flag = true;
					}
				}
				if(update_flag)
				{
					ros::Time current_time = ros::Time::now();
					robot_trajectory.header.stamp = current_time;
					path_stamped.header.stamp=current_time;
					path_stamped.pose.position.x = pose_now.position.x;
					path_stamped.pose.position.y = pose_now.position.y;
					path_stamped.pose.position.z = pose_now.position.z;
					path_stamped.pose.orientation.x = pose_now.orientation.x;
					path_stamped.pose.orientation.y = pose_now.orientation.y;
					path_stamped.pose.orientation.z = pose_now.orientation.z;
					path_stamped.pose.orientation.w = pose_now.orientation.w;
					robot_trajectory.poses.push_back(path_stamped);
				}
			}else ROS_ERROR("ERROR to lookup transform !"); 
				
			path_pub_.publish(robot_trajectory);
			
			rate_.sleep();
		}
	}
	
	
private:
	ros::Publisher path_pub_;
	nav_msgs::Path robot_trajectory;
	
	float sampling_fre;
	float sampling_dist;
	
	std::string robot_link;
	std::string map_link;
	std::string pose_topic;
};
	      
#define USAGE "\nREMIND: You can set the track save parameters in the params.yaml file \n" \
              "  in ${catkin_INCLUDE_DIRS}/path_server/param.\n" 

	      
int main(int argc, char** argv)
{
	ros::init(argc, argv, "path_server", ros::init_options::AnonymousName);
	
	ROS_INFO("%s", USAGE);
	try{
		getPath get_path;
	}catch(std::runtime_error& e)
	{
		ROS_ERROR("map_server exception: %s", e.what());
		return -1;
	}
	
	ROS_INFO("path_server is over......");
	return 0;
}
