#ifndef TOPO_MAPPING_CONSOLE_H
#define TOPO_MAPPING_CONSOLE_H


#include <ros/ros.h>
#include <tf/tf.h>
#include <tf/transform_listener.h>
#include <geometry_msgs/Quaternion.h>
#include <visualization_msgs/MarkerArray.h>
#include <nav_msgs/OccupancyGrid.h>		//ROS中的占有图消息格式
#include <nav_msgs/Path.h>	

#include <iostream>
#include <string>
#include <fstream>

#include <termios.h>
#include <sys/poll.h>

#include "topo_mapping/topology.h"
#include "topo_mapping/mapping.h"
#include "topo_mapping/vertex.h"

//键盘键值
#define KEYCODE_W 0x77	//小写字母w 代号1
#define KEYCODE_A 0x61	//小写字母a 代号2
#define KEYCODE_S 0x73	//小写字母s 代号3
#define KEYCODE_D 0x64	//小写字母d 代号4

#define KEYCODE_W_CAP 0x57	//大写字母W 代号5
#define KEYCODE_A_CAP 0x41	//大写字母A 代号6
#define KEYCODE_D_CAP 0x44	//大写字母S 代号7
#define KEYCODE_S_CAP 0x53	//大写字母D 代号8

#define USAGE "\n\n--------------------------------------------------------------------" \
		"\nKeyboard reminder while using: \nYou can do something by pressing a button. \n" \
              "Each button has a different function:\n" \
		"w: Query vertex information\n" \
		"a: Query topology map information\n" \
		"s: Make changes to the edges of the topology map\n" \
		"d: Adjust the structure of the topology map\n" \
		"W: Build a topology map\n" \
		"A: Growing topological map again\n" \
		"S: Fusion of similar topological regions\n" \
		"D: NONE\n" \
		"--------------------------------------------------------------------\n\n"
		
namespace topo_mapping
{
	//class topology;
	//class mapping;

	class console
	{
	public:
		console();
		~console();
		
		void runPreparation();
		void gridmapCallback(const nav_msgs::OccupancyGridConstPtr& map);
		void robotpathCallback(const nav_msgs::PathConstPtr& path);
		bool vertexs_request(topo_mapping::vertex::Request  &req, topo_mapping::vertex::Response &res);
		int KeyboardDetect(int blockingTime = 50);
		bool KeyEventProcessing();
		void showVertexMarks_rviz();
		void showVertexConnectors_rviz();
		bool mainLoop();
		

	private:
		ros::NodeHandle console_nh;
		ros::Subscriber gridmap_sub, path_sub;
		ros::Publisher vertexMarkers_pub, vertexConnectors_pub;
		ros::ServiceServer vertexs_ser;

		nav_msgs::Path robot_trajectory;
		
		std::string MapFile_path;
		
		//int cascade_times;									//表示级联（多跳连接）的有效次数
		int VertexsQuantity_max;								//拓扑顶点最大数量，用于预先分配内存
		double Res_topo;									//拓扑地图分辨率

		
		struct termios TerminalConfig_ori, TerminalConfig_imp; //保存新配置和原配置
		struct pollfd Terminal_ufd;							//状态事件
		int Terminal_kfd = 0;								//linux下屏幕终端代号
		
		pthread_t showGraph_t; 
		
		topology* TopoMap;			//拓扑地图类，保存着栅格地图数据和拓扑地图数据，以及一些基本的地图操作
		mapping* MappBuilder;		//地图构建类，负责地图的构建工作
		
	};
}
#endif