#ifndef TOPO_MAPPING_CORE_H
#define TOPO_MAPPING_CORE_H

//#include <ros/ros.h>		//不知道为什么bind函数在这个头文件里
#include <boost/bind.hpp>

#include <iostream>
//#include <string>
#include <vector>
#include <algorithm>
#include <functional>
#include <limits.h>
#include <pthread.h>

#include "topo_mapping/topology.h"
#include "topo_mapping/tool.h"

namespace topo_mapping
{
	struct Edge;


	
	class core
	{
	public:
		core();
		~core();

		bool edge_weight_compare(Edge A, Edge B);
		int findAdjEdge(int* parentList, int edge_id);
		void MiniSpanTree_Kruskal(int vextex_maxid, std::vector<Edge>& allEdges, std::vector<Edge>& miniSpanTree);	//使用Kruskal算法获得最小生成树，返回最小生成树的所有边
		void MiniSpanTree_Prim(int vextex_maxid, double** adjEdges, std::vector<Edge>& miniSpanTree);				//使用Prim算法获得最小生成树

		void ImprovedBresenham(cv::Point start, cv::Point end, std::vector<cv::Point>& traversalPoints);				//利用Bresenham划线算法计算两点直线经过的所有点
		
		cv::Point computeCenterOfGravity(cv::Point center, std::vector<cv::Point>& points);							//采用三角形面积法计算不规则多边形的重心
		
		/** 快速凸包算法，计算不规则多边形的凸包
		* @param points 不规则多边形的点集
		* @param p1 是 points 点集中最左边的点
		* @param p2 是 points 点集中最右边的点
		* @return  函数返回 points 点集中位于 向量 p1p2 左边的点集的凸包，以逆时针排列。
		*/
		std::vector<cv::Point> QuickHull(std::vector<cv::Point>& points, cv::Point& p1, cv::Point& p2);
		
		bool point_x_compare(const cv::Point a, const cv::Point b);											//vector容器的sort分类子成员
		void ConvexHull_QuickHull(std::vector<cv::Point>& points, std::vector<cv::Point>& pointsInHull);			//用快速凸包算法求凸包
		
		bool polar_angle_compare(const cv::Point a, const cv::Point b);										//vector容器的sort分类子成员
		bool ymax_xmin_compare(const cv::Point a, const cv::Point b);
		void ConvexHull_GrahamScanning(std::vector<cv::Point> points, std::vector<cv::Point>& pointsInHull);		//Graham扫描法求凸包
		
		bool checkPointInHull(const cv::Point p, const std::vector<cv::Point>& hullPoints);									//利用二分法判断一个点是否在该凸包内
		
		bool pixelSpiralIterator(cv::Point center, int growth_radius, std::vector< cv::Point >& traversalPoints);		//环形迭代器
		
		bool computeEllipseAxes(std::vector<cv::Point>& points, double* longAxe, double* shortAxe);				//利用主成分分析法计算椭圆的长半轴和短半轴
		
	protected:
		cv::Point GrahamStartPoint;
		
		
	private:
		


		
	};
}
#endif