#ifndef TOPO_MAPPING_MAPPING_H
#define TOPO_MAPPING_MAPPING_H

#include <ros/ros.h>
#include <tf/tf.h>
#include <tf/transform_listener.h>
#include <tf/transform_broadcaster.h>
#include <geometry_msgs/Quaternion.h>
#include <visualization_msgs/MarkerArray.h>

#include <opencv2/opencv.hpp>

#include "topo_mapping/core.h"
#include "topo_mapping/topology.h"

namespace topo_mapping
{
	//class topology;

	class mapping : public core
	{
	public:
		mapping(topology* mTopoMap, ros::NodeHandle* nh, bool debug_);
		~mapping();
		
		bool regionsMarkers_pub(int marker_id, std::vector<cv::Point> points);
		
		bool establishATopoVertex(cv::Point2f axesSize = cv::Point2f(0, 0), cv::Point pixelCenter = cv::Point(0, 0), char colorLabel = 0);	//建立一个拓扑节点
		bool getRobotPose(cv::Point2f& pose);										//获取机器人实时位置
		bool statisticsAdjEdges(std::vector<int>& adjEdges, const int locate);		//计算相邻顶点，补充边表，注意是保存的顶点序号，从1开始
		bool statisticsAdjVertexs(std::vector<int>& adjVertexs, const cv::Point2f locate);	//计算规定半径内相邻的顶点，用于指定移动机器人距离最近的顶点
		
		bool addAdjEdges(const int locate);
		bool changePenColor();					//颜色标识号递增操作
		bool adjustmentTopoGraph();				//调整整个拓扑地图结构
		
		bool chooseBestGrowthPoint(cv::Point2f& growthPoint);									//选取最优的生长点
		bool checkBestGrowthPoint(cv::Point2f growthPoint);	
		
		bool pixelDynamicGrowth(cv::Point& pixelenter, cv::Point2f& axesSize, const char colorLabel);		//像素动态生长
		
		bool RegionalRegrowth();
		bool RegionalMerger(const int id, const double area_max, double area_min = 0);					//区域合并
		bool RegionalMerger2(double ratio);					//区域合并
		bool executeRegionsMerger(std::vector<int>& gather, double ratio = 0);					//执行区域合并
		
		void NeighborhoodSegmentation(cv::Point2f pose);		//邻域分割
		
		bool checkPointISQualified(const cv::Point p, const char colorLabel1, const char colorLabel2, char occValue = 0);							//判断一个点是否是合格点
		
		double Radius_max;								//拓扑区域最大半径
		double Radius_min;								//拓扑区域最小半径，也是动态生长的起始半径
		
		pthread_rwlock_t topomap_lock;             //声明读写锁
	private:
		ros::NodeHandle* private_nh;
		ros::Publisher topomap_pub;
		ros::Publisher samples_pub;
		ros::Publisher topo_pointMarkers;
		
		
		char Pen_color;					//画笔颜色，从-120～+120以20递进，作为拓扑区域的颜色标识
		bool debug_flag;
		
		
		double Expansion_size;								//障碍物边界膨胀尺寸
		double ObstacleRatio;								//障碍物比率
		double AddRatio;									//点添加率
		double PassRatio;									//区域点占有率
		double Radius_w1;								//区域半径动态变化控制权值
		
		int AdjacentVertex;								//机器人行进过程中距离最近的顶点标号
		
		topology* maTopoMap;
	};



}
#endif