#ifndef TOPO_MAPPING_TOOL_H
#define TOPO_MAPPING_TOOL_H

#include <iostream>
#include <time.h>
#include <stdlib.h>
//#include <cmath>

#include <opencv2/opencv.hpp>

#include "topo_mapping/topology.h"

namespace topo_mapping
{
	struct Edge;


	class tool
	{
	public:
		tool();
		~tool();
	
		static double getNorm(const cv::Point start_p, cv::Point end_p = cv::Point(0, 0));
		static double getNorm(const cv::Point2f start_p, cv::Point2f end_p = cv::Point2f(0, 0));	//获得两点之间的直线距离(欧几里得距离)
		static double getNorm(const cv::Point2d start_p, cv::Point2d end_p = cv::Point2d(0, 0));
		
		static int getRandI(int low, int high);			//获得low~high之间的随机整数
		static double getRand01();					//获得0~1之间的随机浮点数
		
		static int signum(const int val);										//获得一个数值的正负数情况，正数返回1，负数返回-1,0则返回0
		static double getArea(const cv::Point p1, const cv::Point p2, const cv::Point p3);				//叉乘求三角形面积
		
		
		/*设线段端点为从 p1(x1, y1)到 p2(x2, y2), 线外一点 p3(x0，y0)， 判断该点位于有向线 A→B 的哪一侧。
		a = ( x2-x1, y2-y1)		b = (x0-x1, y0-y1) 
		a x b = | a | | b | sinφ = (x2-x1)(y0-y1) – (y2-y1)(x0-x1)	(φ为两向量的夹角) 
		| a | | b |  ≠ 0 时，  a x b  决定点 P的位置 
		所以  a x b  的 z 方向大小决定 P位置 ：
		a x b  >  0   左侧 
		a x b  <  0   右侧 
		a x b  =  0   线段上 
		 */
		static double ccw(const cv::Point p1, const cv::Point p2, const cv::Point p3);	//采样采用向量叉积(也称外积)方式判断点在直线的哪一侧
		static double dot(const cv::Point a, const cv::Point b);					//向量内积
		static double computePoint2LineDist(cv::Point p, cv::Point a, cv::Point b);	//计算点到线段距离
		
		
	
		

	private:
		
		
		
		
		
		
	};
}
#endif