#include "topo_mapping/core.h"

namespace topo_mapping
{
	core::core()
	{
		GrahamStartPoint = cv::Point(0, 0);
	}
	core::~core()
	{

	}
	bool core::edge_weight_compare(Edge A, Edge B)
	{
		return A.weight < B.weight ? true : false;
	}
	int core::findAdjEdge(int* parentList, int edge_id)
	{
		while(parentList[edge_id] != -1) 
			edge_id = parentList[edge_id];
		
		return edge_id;
	}
	void core::MiniSpanTree_Kruskal(int vextex_maxid, std::vector<Edge>& allEdges, std::vector< Edge >& miniSpanTree)
	{
		auto compare = bind(&core::edge_weight_compare, this,  _1, _2);
		std::sort(allEdges.begin(), allEdges.end(), compare);			//边按从小到大排序
		
		int parent[vextex_maxid+1];								//因为是顶点序号从1开始，首位数并没有使用
		memset(parent, -1, sizeof(parent));
		
		Edge edge_temp;
		int edge_a, edge_b;									//需要判断的两条边
		miniSpanTree.clear();
		for(int i = 0 ; i < allEdges.size() ; i++)						//遍历所有边
		{
// 			int times1, times2;
			edge_a = findAdjEdge(parent, allEdges[i].u);
			edge_b = findAdjEdge(parent, allEdges[i].v);
// 			std::cout<<"MiniSpanTree_Kruskal<1> : "<<(allEdges[i].u)<<" "<<(times1)<<std::endl;
// 			std::cout<<"MiniSpanTree_Kruskal<2> : "<<(allEdges[i].v)<<" "<<(times2)<<std::endl;
			if(edge_a != edge_b) 		//判断结点edges[i].a与edges[i].b是否连接,边与边之间是否形成环路
			{
				parent[edge_a] = edge_b;
				edge_temp.u 		= allEdges[i].u;
				edge_temp.v 		= allEdges[i].v;
				edge_temp.weight = allEdges[i].weight;
				miniSpanTree.push_back(edge_temp);
			}
			
			//判断生成树已连通原图，即已生成最小生成树，提前退出所有边的遍历
			int num = 0;
			for(int j = 0 ; j < vextex_maxid ; j++)
			{
				if(parent[j] != -1 )
					++num;
			}
			if(num == (vextex_maxid - 1) )		//最小生成树的特点是n个节点有n-1条边
				return ;
		}
// 		std::cout<<"MiniSpanTree_Kruskal<3> parent : ";
// 		for(int i = 0 ; i < (vextex_maxid+1) ; i++ )
// 			std::cout<<parent[i]<<" ";
// 		std::cout<<std::endl;
	}
	void core::MiniSpanTree_Prim(int vextex_maxid, double** adjEdges, std::vector< Edge >& miniSpanTree)
	{
		double min_radius = CHAR_MAX;
		
		int adjvexs[vextex_maxid];						//保存相关顶点下标
		double lowcost[vextex_maxid];					//保存相关顶点间的权值
		memset(adjvexs, 1, sizeof(adjvexs));				//初始化全部先为V1的下标
		memset(lowcost, 0, sizeof(lowcost));
		
		//初始化操作，注意V1作为最小生成树的跟开始遍历，权值为0，V1为第一个加入
		for(int i = 2 ; i < vextex_maxid ; i++)
			lowcost[i] = adjEdges[1][i];					//将邻接矩阵第1行所有权值先加入数组
		
		//真正构造最小生成树的过程
		miniSpanTree.clear();
		Edge edge_temp;
		for(int i = 2 ; i < vextex_maxid ; i++)
		{
			//遍历全部顶点
			min_radius = CHAR_MAX;							//初始化权值为255不可能值
			int k = 1;
			for(int j = 2 ; j < vextex_maxid ; j++)
			{	//找出lowcost数组已存储的最小权值
				if(lowcost[j] != 0 && lowcost[j] < min_radius)
				{
					min_radius = lowcost[j];
					k = j;									//将发现的最小权值的下标存入k，以待使用
				}
			}

			//保存当前顶点边中权值最小的边
			edge_temp.u = adjvexs[k];
			edge_temp.v = k;
			edge_temp.weight = min_radius;
			miniSpanTree.push_back(edge_temp);
			lowcost[k] = 0;						//将当前顶点的权值设置为0，表示此顶点已经完成任务，进行下一个顶点的遍历

			//邻接矩阵k行逐个遍历全部顶点，当前最佳顶点权值与上一个最佳顶点权值比较，留下最优权值
			for(int j = 2 ; j < vextex_maxid ; j++)
			{
				if(lowcost[j] != 0 && adjEdges[k][j] < lowcost[j] )
				{
					lowcost[j] 	= adjEdges[k][j];		//权值矩阵更新为最优权值
					adjvexs[j] 	= k;									//连接顶点为当前最佳顶点
				}
			}
		}
	}
	void core::ImprovedBresenham(cv::Point start, cv::Point end, std::vector< cv::Point >& traversalPoints)
	{
		cv::Point temp;		//保存点的临时存储位置
		int dx = 0, dy = 0;		//保存两点之间的横纵坐标距离
		int d = 0;				//误差，点与直线之间的差值
		int incr1=0, incr2=0;	//坐标增长增量
		int x=0, y=0, xend=0, yend=0;//直线增长的首末端点坐标
		int cnt = 0 ;			//直线过点序号
		int xdirflag, ydirflag;	//横纵坐标增长方向
		
		dx = abs(end.x - start.x);
		dy = abs(end.y - start.y);
		
		traversalPoints.clear();
		if(dy <= dx)
		{//斜率k的绝对值|k|<1时，在x方向进行单位步进
			d = dx - 2*dy; 		//初始点P_m0值
			incr1 = - 2 * dy;		//情况（1）
			incr2 = 2 * (dx - dy);	//情况（2）
			if (start.x > end.x)
			{//起点横坐标比终点横坐标大，xdirflag = -1（负号可以理解为增长方向与直线始终点方向相反）
				x = end.x; y = end.y;		//设置增长起点，注意这里要选择小横坐标作为起点，用于增量时好理解
				ydirflag = (-1);				//此时默认(希望)纵坐标也是start.y > end.y的情况
				xend = start.x;			//设置增长终点横坐标
			} else {//xdirflag = 1 ,ydirflag = 1
				x = start.x; y = start.y;
				ydirflag = 1;
				xend = end.x;
			}
			temp.x=x;	//加入起点坐标
			temp.y=y;
			traversalPoints.push_back(temp);
			cnt++;
			if (((end.y - start.y) * ydirflag) > 0)
			{//是预料到的情况，start.y > end.y
				while (x < xend)//开始进行增量递增
				{//x > xend，y > yend，处理向右上角爬升的直线群，下图1号所示直线
					x++;
					if (d > 0)	d+=incr1;
					else {
	 				 y++; d+=incr2;		//纵坐标向正方向增长
					}
					temp.x=x;			//添加新的点
					temp.y=y;
					traversalPoints.push_back(temp);
					cnt++;
				}
			} else {//x > xend，y < yend，处理向右下角降落的直线群，下图2号所示直线
				while (x < xend) 
				{
					x++;
					if (d > 0) 	d+=incr1;
					else {
						y--; d+=incr2;	//纵坐标向负方向增长
					}
					temp.x=x;		//添加新的点
					temp.y=y;
					traversalPoints.push_back(temp);
					cnt++;
				}
			}		
		} else
		{//dy > dx，当斜率k的绝对值|k|>1时，在y方向进行单位步进
			d = dy - 2*dx;			//P_m0初值
			incr1 = -2*dx; 		//形同算法推导情况1
			incr2 = 2 * (dy - dx);
			if (start.y > end.y) 
			{
				y = end.y; x = end.x;	//取最小的纵坐标作为起点
				yend = start.y;
				xdirflag = (-1);			//期望start.x > end.x
			} else {//start.y < end.y
				y = start.y; x = start.x;
				yend = end.y;
				xdirflag = 1;
			}
			temp.x=x;	//加入起点坐标
			temp.y=y;
			traversalPoints.push_back(temp);
			cnt++;
			if (((end.x - start.x) * xdirflag) > 0) 
			{//x > xend ，y > yend，处理向右上角爬升的直线群，下图3号所示直线
				while (y < yend)
				{
					y++;
					if (d > 0) d+=incr1;
					else {
						x++; d+=incr2;		//横坐标向正方向增长
					}
					temp.x=x;		//添加新的点
					temp.y=y;
					traversalPoints.push_back(temp);
					cnt++;
				}
			} else {//x < xend，y > yend，处理向左上角爬升的直线群，下图4号所示直线
				while (y < yend)
				{
					y++;
					if (d > 0) d+=incr1;
					else {
						x--; d+=incr2;	//横坐标向负方向增长
					}
					temp.x=x;		//添加新的点
					temp.y=y;
					traversalPoints.push_back(temp);
					cnt++;
				}
			}
		}
		//删除起点和终点，只要中间经过的点
 		traversalPoints.erase(traversalPoints.end() - 1);
		if(traversalPoints.size() > 1)		//若点集不止一个点，就删除起点
			traversalPoints.erase(traversalPoints.begin());
	}
	cv::Point core::computeCenterOfGravity(cv::Point center, std::vector< cv::Point >& points)
	{
		cv::Point gravity_center(0, 0);
		double area_sum = 0, area = 0;
		int nums = points.size();
		
		for(int i = 1 ; i < nums ; i++)
		{
			area = tool::getArea(center, points[i], points[i - 1]);					//与原点单个三角的面积
			gravity_center.x += (points[i].x + points[i-1].x + center.x) * area;		//重心乘以其权值(面积),因为每一个都要除以3,所以放在最后
			gravity_center.y += (points[i].y + points[i-1].y + center.y) * area;
			area_sum += area;												//计算所有权值
		}
		gravity_center.x = round(gravity_center.x / (area_sum * 3));				//四舍五入取整
		gravity_center.y = round(gravity_center.y / (area_sum * 3));
		
		return gravity_center;
	}
	std::vector< cv::Point > core::QuickHull(std::vector< cv::Point >& points, cv::Point& p1, cv::Point& p2)
	{
		std::vector<cv::Point> result;
		int points_size = points.size();
		int id = -1;
		double max_dist = 0;
		
		//寻找距离p1p2线段最远的点id
		for(int i = 0 ; i < points_size; i++)
		{
			cv::Point  temp = points[i];
			double dist = tool::computePoint2LineDist(temp, p1, p2);
			if(tool::ccw(p1, p2, temp) > 0)// ret > 0 : temp在p1p2左侧
			{
				if(dist > max_dist)
				{
					max_dist = dist;
					id = i;
				}
			}
		}
		result.push_back(p2);
		if(id != -1)
		{
			cv::Point p3 = points[id];
			std::vector<cv::Point> left_points, right_points;
			left_points.clear();
			right_points.clear();
			for(int i = 0 ; i < points_size ; i++)
			{
				cv::Point temp = points[i];
				if(tool::ccw(p1, p3, temp) > 0)
					left_points.push_back(temp);
				if(tool::ccw(p3, p2, temp) > 0)
					right_points.push_back(temp);
			}
			
			std::vector<cv::Point> left = QuickHull(left_points, p1, p3);
			std::vector<cv::Point> right = QuickHull(right_points, p3, p2);
			
			if(right.size() > 2)
				result.insert(result.end(), right.begin() + 1, right.end() - 1);
			result.push_back(p3);
			if(left.size() > 2)
				result.insert(result.end(), left.begin() +1, left.end() -1);
		}
		result.push_back(p1);
		return result;
	}
	bool core::point_x_compare(const cv::Point a, const cv::Point b)
	{
		return a.x < b.x ? true : false;
	}
	void core::ConvexHull_QuickHull(std::vector< cv::Point >& points, std::vector< cv::Point >& pointsInHull)
	{
		pointsInHull.clear();
		
		auto compare = bind(&core::point_x_compare, this,  _1, _2);
		std::sort(points.begin(), points.end(), compare);			//按点横坐标从小到大排列
		
		cv::Point a = points[0];
		cv::Point b = points[points.size() - 1];
		pointsInHull = QuickHull(points, a, b);						//求得不规则多边形的凸包的上半部分
		pointsInHull.pop_back();
		std::vector<cv::Point> lower_half = QuickHull(points, b, a);	//求得不规则多边形的凸包的下半部分
		pointsInHull.insert(pointsInHull.end(), lower_half.begin(), lower_half.end());
		pointsInHull.pop_back();
	}
	bool core::polar_angle_compare(const cv::Point a, const cv::Point b)
	{
		double temp = tool::ccw(GrahamStartPoint, a, b);
		if (fabs(temp) < 1e-6)//极角相等按照距离从小到大排序
		{
			return tool::getNorm(GrahamStartPoint, a) < tool::getNorm(GrahamStartPoint, b);
		}else{
			return temp > 0;
		}
	}
	bool core::ymax_xmin_compare(const cv::Point a, const cv::Point b)
	{
		if(a.y == b.y)
			return a.x < b.x ? true:false;
		else
			return a.y > b.y ? true:false;
	}
	void core::ConvexHull_GrahamScanning(std::vector< cv::Point > points, std::vector< cv::Point >& pointsInHull)
	{
		//查找起始点，左下
		int index = 0;
		cv::Point p_start = points[0];
		for(int i = 0 ; i < points.size() ; i++)
		{//选出Y坐标最大的点，若Y坐标相等，选择X坐标小的点
			if (points[i].y > p_start.y || (points[i].y == p_start.y && points[i].x < p_start.x))
			{
				p_start = points[i];
				index = i;
			}
		}
		std::swap(points[0], points[index]);
// 		points[index] = points[0];
// 		points[0] = p_start;
// 		auto compare1 = bind(&core::ymax_xmin_compare, this,  _1, _2);
// 		std::sort(points.begin() + 1, points.end(), compare1);			//坐标排序，选取左下角x坐标最小y坐标最大的点
		GrahamStartPoint =  points[0];// 保存起始点

		auto compare2 = bind(&core::polar_angle_compare, this,  _1, _2);
		std::sort(points.begin() + 1, points.end(), compare2);			//极角排序，若相等按照距离从小到大排序
		
		int top = 2;
		pointsInHull.clear();
		pointsInHull.push_back(GrahamStartPoint);
		pointsInHull.push_back(points[1]);
		pointsInHull.push_back(points[2]);
		for (int i = 3 ; i < points.size() ; ++i)
		{
			while (top > 0 && tool::ccw(pointsInHull[top - 1], points[i], pointsInHull[top]) >= 0)
			{
				--top;
				pointsInHull.pop_back();
			}
			pointsInHull.push_back(points[i]);
			++top;
		}
	}
	bool core::checkPointInHull(const cv::Point p, const std::vector< cv::Point >& hullPoints)
	{
		//这里尤其要注意ccw函数判断条件在这里是相反的，因为
		//(1)凸包点起点在最右端或者最上端，顺时针排序
		//(2)这里的点坐标是图像坐标，向右为x轴正向，向下为y轴正向
		//所以，ccw>0时表示点在直线右边；ccw<0时在左边
		int left = 2, right = hullPoints.size() - 1, middle;
		
		//在第一个点为起点,第二个为终点和最后一个为终点的两个向量构成的扇形之外
		if(tool::ccw(hullPoints[0], hullPoints[1], p) <= 0 || tool::ccw(hullPoints[0], hullPoints[right], p) >= 0)
			return false;
		
		//进行二分法查找，找到点所在的三角形区域
		while(left < right)
		{
			middle = (left + right)/2;
			if(tool::ccw(hullPoints[0], hullPoints[middle], p) < 0)//点在中线的左边
				right = middle;
			else
				left = middle + 1;
		}
		
		//在边界矢量的左边即为“界外“，右边即为”界内“
		if(tool::ccw(hullPoints[left-1], hullPoints[left], p) > 0)
			return true;
		else
			return false;
	}
	bool core::pixelSpiralIterator(cv::Point center, int growth_radius, std::vector< cv::Point >& traversalPoints)
	{
		cv::Point point, pointInMap, normal;
		
		point.x = growth_radius;
		point.y = 0;
		
		traversalPoints.clear();
		if(growth_radius == 1)//第一次遍历将中心点加入点集
			traversalPoints.push_back(center);

		do{
			pointInMap.x = center.x + point.x;
			pointInMap.y = center.y + point.y;
			traversalPoints.push_back(pointInMap);
			
			normal.x = -tool::signum(point.y);
			normal.y = tool::signum(point.x);
			if(normal.x != 0 && ((int)tool::getNorm(cv::Point(point.x + normal.x, point.y))) == growth_radius)
				point.x += normal.x;
			else if(normal.y != 0 && ((int)tool::getNorm(cv::Point(point.x, point.y + normal.y))) == growth_radius)
				point.y += normal.y;
			else{
				point.x += normal.x;
				point.y += normal.y;
			}
		}while(point.x != growth_radius || point.y != 0);
		
		return traversalPoints.empty() ? false:true;
	}
	bool core::computeEllipseAxes(std::vector< cv::Point >& points, double* longAxe, double* shortAxe)
	{
		int point_num = points.size();

		cv::Mat points_mat(point_num, 2, CV_32FC1);
		for(int i = 0 ; i < point_num ; i++)
		{
			points_mat.at<float>(i, 0) = points[i].x;
			points_mat.at<float>(i, 1) = points[i].y;
		}
		
// 		float Coordinates[20]={2.5, 2.4,  
// 							0.5, 0.7,  
// 							2.2, 2.9,  
// 							1.9, 2.2,  
// 							3.1, 3.0,  
// 							2.3, 2.7,  
// 							2.0, 1.6,  
// 							1.0, 1.1,  
// 							1.5, 1.6,  
// 							1.1, 0.9};
// 		cv::Mat points_mat(10, 2, CV_32FC1, Coordinates);
		
		cv::PCA pca_tool(points_mat, cv::noArray(), CV_PCA_DATA_AS_ROW, 2);
		
// 		std::cout<<"eigenvalues="<<pca_tool.eigenvalues<<std::endl;
// 		std::cout<<"eigenvectors="<<pca_tool.eigenvectors<<std::endl;
		
		cv::Mat points_dst=pca_tool.project(points_mat);			//计算特征向量，并将源数据对该向量进行投影
// 		std::cout<<"projected mat:"<<std::endl;
// 		std::cout<<points_dst<<std::endl;
		
		double axe_start1, axe_start2, axe_end1, axe_end2;
		cv::minMaxIdx(points_dst.col(0), &axe_start1, &axe_end1);
		cv::minMaxIdx(points_dst.col(1), &axe_start2, &axe_end2);
// 		std::cout<<"axe_start1 = "<<(axe_start1)<<" axe_end1 = "<<axe_end1<<std::endl;
// 		std::cout<<"axe_start2 = "<<(axe_start2)<<" axe_end2 = "<<axe_end2<<std::endl;
		
		*longAxe 	= fabs(axe_start1 - axe_end1) / 2.0;
		*shortAxe 	= fabs(axe_start2 - axe_end2) / 2.0;
// 		std::cout<<"longAxe = "<<(*longAxe)<<std::endl;
// 		std::cout<<"shortAxe = "<<(*shortAxe)<<std::endl;
	}



}
