#include "topo_mapping/topology.h"

namespace topo_mapping
{

	topology::topology(double res, int vertexs_res, int vertexs_size, bool debug)
		:Vertex_maxsize(vertexs_size), Res_topo(res),debug_flag(debug)
	{

		VertexsList.clear();
		VertexsList.reserve(vertexs_res);
		AbandonedVertexs.clear();
		
		VertexesNum = 0;
		EdgesNum = 0;
		AdjMatrix_vertexs = 0;
		AdjMatrixVertexs = NULL;
		AdjMatrixEdges = NULL;
		
		Vertex_id = 1;				//序号从1开始
		
		Res_grid = 0;
		Orign_grid = cv::Point2f(0, 0);
		
		GridMap_temp.data.resize(0);
		TopoMap_temp.data.resize(0);
		
		VertexIter_t = VertexsList.end();
	}
	topology::~topology()
	{
		showTopoMap_graph();
		free(AdjMatrixVertexs);
		free(AdjMatrixEdges);
	}
	int topology::edge_size()
	{
		return EdgesNum;
	}
	double topology::getResolution(bool isIntopo)
	{
		//return GridMap_temp.info.resolution;
		return isIntopo == false ? Res_grid:Res_topo;
	}
	cv::Point2f topology::getOrignposeInmap()
	{
		return Orign_grid;
	}
	int topology::vertex_size()
	{
		return VertexesNum;
	}
	double** topology::getAdjMatrix()
	{
		return AdjMatrixEdges;
	}
	inline  bool topology::setVertexIndex(const int index)
	{
//		Vertex_id = index;
// 		if(Vertex_id < Vertex_maxsize)
// 			return true;
// 		else
// 			return false;
		return index<Vertex_maxsize ? true:false;
	}
	bool topology::setTopoMapLabel(const cv::Point pixel_pos, const char label)
	{
		int linear_index = pixel2linearIndex(pixel_pos, true);
		if(linear_index < TopoMap_temp.data.size())
		{
			TopoMap_temp.data[linear_index] = label;
			return true;
		}else{
			std::cerr<<"An error occurred in the cross-border access topomap data !"<<std::endl;
			return false;
		}
	}
	void topology::setAllEdgeInvalid()
	{
		VertexNode* vertex_temp;
		
		for(int i = 0 ; i < VertexesNum ; i++)
		{
			vertex_temp = &(VertexsList[i]);
			for(int j = 0 ; j < vertex_temp->adjEdges.size() ; j++)
				vertex_temp->adjEdges[j].status = false;
		}
	}
	bool topology::updateAdjMatrix()
	{
		float dis_max = CHAR_MAX;
		int vertex_num = VertexesNum+ 1 ;			//由于顶点序号从1开始编号，而数组从0开始索引
		
		if(AdjMatrix_vertexs != VertexesNum )
		{
			AdjMatrix_vertexs 	= vertex_num - 1;

			//提前释放内存资源
			if(AdjMatrixVertexs != NULL) 
			{
				free(AdjMatrixVertexs);
				free(AdjMatrixEdges);
			}
			
			//申请顶点索引表相关资源，并进行初始化，此处的一维数组后面用不到
			AdjMatrixVertexs		= (int*)malloc(sizeof(int)*vertex_num);
			if(!AdjMatrixVertexs) 
			{
				std::cerr<<"Failed to apply for memory of vertexs in AdjMatrix !"<<std::endl;
				return false;
			}
			for(int i = 0 ; i < vertex_num ; i++)
				AdjMatrixVertexs[i] = i;

			//申请邻接矩阵所占内存资源，并进行初始化
			AdjMatrixEdges = (double**)malloc(sizeof(double*)*vertex_num);
			if(!AdjMatrixEdges) 
			{
				std::cerr<<"Failed to apply for memory of first col edges in AdjMatrix !"<<std::endl;
				return false;
			}
			double **temp = AdjMatrixEdges;
			for(int i = 0 ; i < vertex_num ; i++)//二维数组(各边的权值)初始化
			{
				*temp=(double*)malloc(sizeof(double)*vertex_num);
				if( !*temp ) 
				{
					std::cerr<<"Failed to apply for memory of first row edges in AdjMatrix !"<<std::endl;
					return false;
				}
				++temp;			
			}
			for(int i=0 ; i < vertex_num ; i++)
				for(int j =0 ; j < vertex_num ; j++)
				{
					if(i == j)
						AdjMatrixEdges[i][j] = 0;
					else
						AdjMatrixEdges[i][j] = dis_max;
				}
				

			//将拓扑地图信息导入邻接矩阵
			VertexNode* vertex_temp;
			for(int i = 0 ; i < (vertex_num-1) ; i++)
			{
				vertex_temp = &(VertexsList[i]);
				for(int j = 0 ; j < vertex_temp->adjEdges.size() ; j++)
					AdjMatrixEdges[vertex_temp->index][vertex_temp->adjEdges[j].adjvex] = vertex_temp->adjEdges[j].weight;
			}
		}
	}
	bool topology::vertex_id_compare(VertexNode a, VertexNode b)
	{
		return a.index <b.index ? true:false;
	}
	int topology::insertAVertex(const VertexNodeInfo msg, char center_label)
	{
		struct VertexNode temp;
		int index = 0;
		
		if(VertexesNum != 0)	//非初始状态
		{
			if(AbandonedVertexs.empty() == false)
			{
				index = AbandonedVertexs.front();	//从废弃顶点序号池中选取序号使用
				AbandonedVertexs.pop_front();
			}else{
				index = Vertex_id;
				index++;
				Vertex_id = index;
			}
			
		}else
			index++;
		
		temp.index = index;
		temp.edges = 0;
		temp.adjEdges.clear();
		temp.data.rgb_label		= msg.rgb_label;
		temp.data.axes_size		= msg.axes_size;
// 		temp.data.pose 			= msg.pose;
// 		temp.data.center			= map2pixelCoord(msg.pose, true);
		temp.data.center 			= msg.center;
		temp.data.pose			= pixel2mapCoord(msg.center, true);
		temp.data.intention 		= msg.intention;
			
		//setTopoMapLabel(map2pixelCoord(msg.pose, true), center_label);//设置拓扑区域中心点颜色标识
		
		
		
		if(index < (VertexsList.back()).index)
		{
			VertexsList.push_back(temp);
			auto compare = bind(&topology::vertex_id_compare, this,  _1, _2);
			std::sort(VertexsList.begin(), VertexsList.end(), compare);			//按id从小到大排列
		}else
			VertexsList.push_back(temp);
		VertexesNum++;				//节点个数加1
		
		if(debug_flag)
			std::cout<<"topology: "<<(index)<<"号顶点插入完毕, 使用颜色: "<<(int)(msg.rgb_label)<<"  半径: "<<(msg.axes_size.x)<<" - "<<(msg.axes_size.y)<<std::endl;
		return index;
	}
	bool topology::addAnEdge(const int vertex2, int vertex1)
	{
		struct EdgeNode edge_temp;
		edge_temp.adjvex = vertex2;
		edge_temp.status = true;
		
		if(vertex1 == 0)			//若是0，说明是默认情况
			vertex1 = Vertex_id;
		
		VertexNode* vertex_temp;
		for(int i = 0 ; i < VertexesNum ; i++)
		{
			vertex_temp = &(VertexsList[i]);
			if(vertex_temp->index == vertex1)
			{
				//检查是否重复添加
				for(int j = 0 ; j < vertex_temp->adjEdges.size() ; j++)
				{
					if(vertex_temp->adjEdges[j].adjvex == vertex2)
						return true;
				}
				cv::Point2f end = (getVertexNode(vertex2))->data.pose;
				edge_temp.weight = tool::getNorm(vertex_temp->data.pose, end);
				vertex_temp->adjEdges.push_back(edge_temp);
				vertex_temp->edges++;
				break;
			}
		}	
		EdgesNum++;
		return true;
	}
	bool topology::deleteAnEdge(const int vertex2, int vertex1)
	{
		int check = 0;
		if(vertex1 == 0)			//若是0，说明是默认情况
			vertex1 = Vertex_id;
		
		std::vector<VertexNode>::iterator iter_v;
		std::vector<EdgeNode>::iterator iter_e;
		for(iter_v = VertexsList.begin() ; iter_v != VertexsList.end() ; ++iter_v)
		{
			if((*iter_v).index == vertex1 || (*iter_v).index == vertex2)
			{
				for(iter_e = (*iter_v).adjEdges.begin() ; iter_e != (*iter_v).adjEdges.end() ; )
				{
					if((*iter_e).adjvex == vertex2 || (*iter_e).adjvex == vertex1)
					{
						iter_e =  (*iter_v).adjEdges.erase(iter_e);		//删除指定边表
						(*iter_v).edges--;							//顶点边表数目减一
						EdgesNum--;								//无向图总边表数减一
						check++;
						break;
						//return ;
					}else
						++iter_e;
				}
				if(check == 2)
					break;
			}
		}
		if(check == 2)
			return true;
		else 
			return false;
	}
	bool topology::deleteAVertex(const int vertex_index, bool deletelabel)
	{
		VertexNodeInfo vertex_d;			//删除的顶点的相关信息
		std::vector<int> delete_indexs;
		std::vector<VertexNode>::iterator iter_v;
		std::vector<EdgeNode>::iterator iter_e;
		
		//删除顶点数据
		delete_indexs.clear();
		for(iter_v = VertexsList.begin() ; iter_v != VertexsList.end() ; )
		{
			if((*iter_v).index == vertex_index)
			{
				for(iter_e = (*iter_v).adjEdges.begin() ; iter_e != (*iter_v).adjEdges.end() ; ++iter_e)
				{
					delete_indexs.push_back((*iter_e).adjvex);
					EdgesNum--;
				}
				vertex_d = (*iter_v).data;
				iter_v = VertexsList.erase(iter_v);		//删除顶点数据
				VertexesNum--;
				break;
			}else
				++iter_v;
		}
		
		//删除相邻顶点的相关边表
		std::vector<int>::iterator iter_d;
		if(!delete_indexs.empty())			//如果边表不为空，则进行查找并删除
		{
			for(iter_v = VertexsList.begin() ; iter_v != VertexsList.end() ; ++iter_v)
			{
				for(iter_d = delete_indexs.begin() ; iter_d != delete_indexs.end() ; )
				{
					if((*iter_v).index == (*iter_d))
					{
						for(iter_e = (*iter_v).adjEdges.begin() ; iter_e != (*iter_v).adjEdges.end() ; )
						{
							if((*iter_e).adjvex == vertex_index)
							{
								iter_e =  (*iter_v).adjEdges.erase(iter_e);		//删除指定边表
								(*iter_v).edges--;							//顶点边表数目减一
								EdgesNum--;								//无向图总边表数减一
								break;
							}else
								++iter_e;
						}
						iter_d = delete_indexs.erase(iter_d);
						break;
					}else
						++iter_d;
				}
			}
		}
		
		AbandonedVertexs.push_back(vertex_index);		//将删除顶点序号扔入废弃池中
		AbandonedVertexs.sort();
	}
	int topology::deleteFalseEdges()
	{
		int num = 0 ;
		VertexNode* vertex_temp;
		//删除无效边
		std::vector< EdgeNode >::iterator iter;
		for(int i = 0 ; i < VertexesNum ; i++)
		{
			vertex_temp = &(VertexsList[i]);
			for(iter = vertex_temp->adjEdges.begin() ; iter != vertex_temp->adjEdges.end(); )
			{
				if((*iter).status == false)
				{
					iter = vertex_temp->adjEdges.erase(iter);			//删除指定边表
					vertex_temp->edges--;							//顶点边表数目减一
					EdgesNum--;									//无向图总边表数减一
					num++;
				}else
					++iter;
			}
		}
		return num;
	}
	double topology::computeOccupiedRate()
	{
		unsigned int free_space_num = 0;
		unsigned int topo_space_num = 0;
		
		for(unsigned int y = 0; y < GridMap_temp.info.height; y++)
		{
			for(unsigned int x = 0; x < GridMap_temp.info.width; x++) 
			{
				unsigned int i = pixel2linearIndex(cv::Point(x, y));
				if (GridMap_temp.data[i] == 0) 
				  free_space_num++;
				if (TopoMap_temp.data[i] != 0) 
				  topo_space_num++;
			}
		}
	  
		return ((double)topo_space_num/(double)free_space_num);
	}
	bool topology::updateGridMap(const nav_msgs::OccupancyGridConstPtr& map)
	{
		if(debug_flag)
			std::cout<<"topology : Received a "<<(map->info.width)<<" X "<<(map->info.height)<<" gridmap @ "<<(map->info.resolution)<<" m/pix"<<std::endl;
		
		//初始化拓扑地图参数，并保证临时栅格地图
		GridMap_temp.header.stamp		 = map->header.stamp;
		GridMap_temp.header.frame_id 		= map->header.frame_id;
		GridMap_temp.info.map_load_time 	= ros::Time::now();
		GridMap_temp.info.width 			= map->info.width;
		GridMap_temp.info.height 			= map->info.height;
		GridMap_temp.info.resolution 		= map->info.resolution;
		GridMap_temp.info.origin 			= map->info.origin;
		
		GridMap_temp.data.resize(map->info.width * map->info.height);
		GridMap_temp.data 				= map->data;
		
		Res_grid = map->info.resolution;
		Orign_grid.x = map->info.origin.position.x;
		Orign_grid.y = map->info.origin.position.y + map->info.height * map->info.resolution;
	}
	bool topology::updateTopoMap(const nav_msgs::OccupancyGridConstPtr& map, bool needdata)
	{
		if(debug_flag)
			std::cout<<"topology: Received a "<<(map->info.width)<<" X "<<(map->info.height)<<" gridmap @ "<<(map->info.resolution)<<" m/pix"<<std::endl;
		
		//初始化拓扑地图参数，并保证临时栅格地图
		TopoMap_temp.header.stamp		 = map->header.stamp;
		TopoMap_temp.header.frame_id 	= map->header.frame_id;
		TopoMap_temp.info.map_load_time 	= ros::Time::now();
		TopoMap_temp.info.origin 			= map->info.origin;
		
		double magn = (map->info.resolution) / Res_topo ;			//分辨率倍率，可以理解为缩小比率
		TopoMap_temp.info.width 			= (map->info.width) * magn;
		TopoMap_temp.info.height 			= (map->info.height) * magn;
		TopoMap_temp.info.resolution 		= Res_topo;
		Orign_topo.x = map->info.origin.position.x;
		Orign_topo.y = map->info.origin.position.y + map->info.height * map->info.resolution;
		
		static bool init_flag = false;
		if(init_flag == false)
		{
			TopoMap_temp.data.resize(TopoMap_temp.info.width * TopoMap_temp.info.height);
			init_flag = true;
		}
		
		if(needdata == true)
			TopoMap_temp.data = map->data;
	}
	void topology::updateMapTime()
	{
		TopoMap_temp.info.map_load_time 	= ros::Time::now();
	}
	bool topology::isGridMapEmpty()
	{
		return GridMap_temp.data.empty();
	}
	bool topology::isTopoMapEmpty()
	{
		return TopoMap_temp.data.empty();
	}
	nav_msgs::OccupancyGrid* topology::getTopoMap()
	{
		return &(TopoMap_temp);
	}
	char topology::getOccValue(const cv::Point point)
	{
		int linear_index = pixel2linearIndex(point);
		if(linear_index < GridMap_temp.data.size())
		{
			return GridMap_temp.data[linear_index];
		}else{
			std::cerr<<"An error occurred in the cross-border access gridmap data !"<<std::endl;
			return 0;
		}
	}
	char topology::getTopoLabel(const cv::Point point)
	{
		int linear_index = pixel2linearIndex(point, true);
		if(linear_index < TopoMap_temp.data.size())
		{
			return TopoMap_temp.data[linear_index];
		}else{
			std::cerr<<"An error occurred in the cross-border access topomap data !"<<std::endl;
			return 0;
		}
	}
	VertexNode* topology::getVertexNode(const int locate)
	{
		VertexNode* result = NULL ;
		
		if(locate <= (VertexsList.back()).index)	//索引号合法
		{
// 			std::cerr<<"This is a wrong node index number ! "<<std::endl;
// 			return result;
			if(VertexsList[locate - 1].index == locate)//提前返回，节省时间
				return (&(VertexsList[locate - 1]));
		}
		
		for(int i = 0 ; i < VertexesNum ; i++)
		{
			if(VertexsList[i].index == locate)
			{
				result = &(VertexsList[i]);
				break ;
			}
		}
		return result;
	}
	void topology::getAllEdges(std::vector< Edge >& allEdges)
	{
		int points_num = 0;
		int edge = 0;
		Edge edge_temp;
		
		//FIXME 这里的方法不完美，会造成空间浪费，需修改
		if(VertexesNum != 0)
			points_num = (VertexsList.back()).index + 1;
		
		bool AdjacencyMatrix[points_num][points_num];
		memset(AdjacencyMatrix, false, sizeof(AdjacencyMatrix));
		
		allEdges.clear();
		std::vector<VertexNode>::iterator vertex_iter;
		for(vertex_iter = VertexsList.begin() ; vertex_iter != VertexsList.end() ; ++vertex_iter)
		{
			//准备遍历邻接节点
			edge_temp.u = (*vertex_iter).index;
			for(int j = 0 ; j <  (*vertex_iter).adjEdges.size() ; j++)
			{
				edge =  (*vertex_iter).adjEdges[j].adjvex;
				if((AdjacencyMatrix[ (*vertex_iter).index][edge] == false)&&(AdjacencyMatrix[edge][ (*vertex_iter).index] == false))
				{
					edge_temp.v = edge;
					edge_temp.weight =  (*vertex_iter).adjEdges[j].weight;
					allEdges.push_back(edge_temp);
					AdjacencyMatrix[ (*vertex_iter).index][edge] = true;
				}
			}
		}
	}
	VertexNode* topology::getLastVertexInfo()
	{
		return &((VertexsList.back()));
	}
	void topology::resetVertexIter()
	{
		vertexItertemp();				//检查迭代器是否需要临时保存
		VertexIter = VertexsList.begin();
	}
	std::vector< VertexNode >::iterator topology::getVertexIter()
	{
		return VertexIter;
	}
	bool topology::vertexIterIsEnd()
	{
		++VertexIter;
		if(VertexIter == VertexsList.end())
		{
			vertexItertemp();			//恢复临时保存的迭代器
			return true;
		}else
			return false;
	}
	bool topology::vertexItertemp()
	{
		
		if(VertexIter_t == VertexsList.end())
		{
			if(VertexIter == VertexsList.end())
				return false;
			VertexIter_t = VertexIter;
		}else{
			VertexIter = VertexIter_t;
			VertexIter_t = VertexsList.end();
		}
		return true;
	}
	std::vector< VertexNode >* topology::getVertexsList()
	{
		return &VertexsList;
	}
	cv::Point topology::map2pixelCoord(const cv::Point2f mapCoord, bool isIntopo)
	{
		cv::Point pixel_coord = cv::Point(0, 0);
		double res;
		
		if(isIntopo == false)
			res = Res_grid;
		else
			res = Res_topo;
		
		if(res != 0)
		{
			pixel_coord.x = abs(int(  ((mapCoord.x) - Orign_grid.x) /(res) ));
			pixel_coord.y = abs(int(  ((mapCoord.y) - Orign_grid.y) /(res) ));
		}else
			std::cerr<<"The map parameters are not initialized ! "<<std::endl;
		
		return pixel_coord;
	}
	cv::Point2f topology::pixel2mapCoord(const cv::Point pixelCoord, bool isIntopo)
	{
		cv::Point2f map_coord = cv::Point2f(0, 0);
		double res;
		
		if(isIntopo == false)
			res = Res_grid;
		else
			res = Res_topo;
		
		if(res != 0)
		{
			map_coord.x = Orign_grid.x + (pixelCoord.x * res) + res/2.0;
			map_coord.y = Orign_grid.y - (pixelCoord.y * res) - res;
// 			map_coord.x = Orign_grid.x + (pixelCoord.x * res);
// 			map_coord.y = Orign_grid.y - (pixelCoord.y * res);
		}else
			std::cerr<<"The map parameters are not initialized ! "<<std::endl;
		
		return map_coord;
	}
	int topology::pixel2linearIndex(const cv::Point pixelCoord, bool isIntopo)
	{
		int sx, sx_i, sx_j;
		
		sx_i = pixelCoord.x;
		if(isIntopo == false)
		{
			sx = GridMap_temp.info.width;
			sx_j = GridMap_temp.info.height - pixelCoord.y - 1;
		}else{
			sx = TopoMap_temp.info.width;
			sx_j = TopoMap_temp.info.height - pixelCoord.y - 1;
		}
		
		return ((sx) * (sx_j) + (sx_i));
	}
	void topology::showTopoMap_graph()
	{
		std::cout<<"****************************************************** "<<std::endl;
		std::cout<<"现在打印拓扑结构图, 该拓扑图有 "<< (VertexesNum) <<" 个拓扑节点，有 "<<(EdgesNum)<<" 条边."<<std::endl;
		std::cout<<"顶点序号            边表1      边表2      ......"<<std::endl;
		std::cout<<"****************************************************** "<<std::endl;
		
		std::vector<VertexNode>::iterator vertex_iter;
		for(vertex_iter = VertexsList.begin() ; vertex_iter != VertexsList.end() ; ++vertex_iter)
		{
			//输出拓扑节点
			std::cout<<"[";
			std::cout<<(*vertex_iter).index;
			std::cout<<"](";
			std::cout.setf(std::ios::fixed);
			std::cout.setf(std::ios::showpoint);
			std::cout.precision(2);
			std::cout<<(*vertex_iter).data.pose.x;
			std::cout<<",";
			std::cout<<(*vertex_iter).data.pose.y;
			std::cout<<")  ";
			
			//输出邻接节点序号
			for(int j = 0 ; j < (*vertex_iter).edges ; j++)
				std::cout<<((*vertex_iter).adjEdges[j].adjvex)<<" ("<<((*vertex_iter).adjEdges[j].weight)<<") ";
			
			std::cout<<" NULL "<<std::endl;
		}
		std::cout<<"****************************************************** "<<std::endl;
	}
	void topology::showVertexData()
	{
		std::cout<<"****************************************************** "<<std::endl;
		std::cout<<"现在打印各个顶点的详细信息, 该拓扑结构图有 "<< (VertexesNum) <<" 个拓扑节点. "<<std::endl;
		std::cout<<"序号             短半轴 长半轴 面积 邻域标识 意图   "<<std::endl;
		std::cout<<"****************************************************** "<<std::endl;
		
		std::vector<VertexNode>::iterator vertex_iter;
		for(vertex_iter = VertexsList.begin() ; vertex_iter != VertexsList.end() ; ++vertex_iter)
		{
			//输出拓扑节点
			std::cout<<"[";
			std::cout<<(*vertex_iter).index;
			std::cout<<"](";
			std::cout.setf(std::ios::fixed);
			std::cout.setf(std::ios::showpoint);
			std::cout.precision(2);
			std::cout<<(*vertex_iter).data.pose.x;
			std::cout<<",";
			std::cout<<(*vertex_iter).data.pose.y;
			std::cout<<")  ";
			
			//输出节点的相关信息
			double area = M_PI * ((*vertex_iter).data.axes_size.x) * ((*vertex_iter).data.axes_size.y);
			std::cout<<((*vertex_iter).data.axes_size.x)<<"   "<<((*vertex_iter).data.axes_size.y)<<"   "<<(area)<<"   "<<(int)((*vertex_iter).data.rgb_label)<<" "<<((*vertex_iter).data.intention); 
			
			std::cout<<" NULL "<<std::endl;
		}
		std::cout<<"****************************************************** "<<std::endl;
	}
	bool topology::saveTopoMap2TXT(std::string path)
	{
		time_t t_source;
		time( &t_source );
		t_source = t_source + 8*3600;  		// transform the time zone
		tm* t_now= gmtime( &t_source );	//得到系统时间结构体
		
		//是否允许保存拓扑图
		std::string temp;
		std::cout<<"当前的拓扑图是否需要保存?(y/n or Y/N)";
		std::cin>>temp;
		if(temp == "n" || temp == "N")
		{
			std::cout<<"不保存拓扑图，即将退出......"<<std::endl;
			return false;
		}
		
		std::string file_name, file_check;
		file_name = path+"topomap_"+std::to_string( t_now->tm_mon + 1 )+std::to_string(t_now->tm_mday)+"_";
		file_check = "topomap_"+std::to_string( t_now->tm_mon + 1 )+std::to_string(t_now->tm_mday);
		
		//检查同名文件
		struct dirent *ptr;
		DIR *dir;
		std::string file;
		int index=0;
		dir=opendir(path.c_str());
		while((ptr=readdir(dir))!=NULL)
		{
			//跳过'.'和'..'两个目录
			if(ptr->d_name[0] == '.')
				continue;
			file = ptr->d_name;
			if(file.find(file_check) != std::string::npos)
			{
				//提取同名文件的序列号
				file.erase(0, file_check.length()+1);
				file.erase(file.find( '.'), 4);
				int file_index = atoi(file.c_str());
				if(   file_index > index)
					 index = file_index; 
			}
		}
		
		if(index > 0)
		{//有同名文件
			file_name = file_name + std::to_string(index+1) +".txt";
		}else{//没有同名文件，默认从1开始编号
			file_name = file_name + "1.txt";
		}

		std::ofstream  file_output;			//用于向文件写数据（输出到文件）
		file_output.open(file_name);
		if(file_output.is_open())
		{
			struct EdgeNode* nextEdge;
			
			file_output<< VertexesNum <<std::endl;
			std::vector<VertexNode>::iterator vertex_iter;
			for(vertex_iter = VertexsList.begin() ; vertex_iter != VertexsList.end() ; ++vertex_iter)
			{
				//输出拓扑顶点信息
				file_output<< (*vertex_iter).index<<" ";
				file_output.setf(std::ios::fixed);
				file_output.setf(std::ios::showpoint);
				file_output.precision(2);
				file_output<< (*vertex_iter).data.pose.x<<" ";
				file_output<< (*vertex_iter).data.pose.y<<" ";
				
				//输出邻接节点序号
				for(int j = 0 ; j < (*vertex_iter).edges ; j++)
					file_output<<((*vertex_iter).adjEdges[j].adjvex)<<" ";

				file_output<<"NULL ";
				file_output<< (*vertex_iter).data.intention<<std::endl;;
			}
			file_output<<"拓扑地图构建于："<<( t_now->tm_year + 1900 )<<"年"<<( t_now->tm_mon + 1 )<<"月"<< (t_now->tm_mday)<<"日";
			file_output<<( t_now->tm_hour )<<"时"<<(t_now->tm_min)<<"分"<<(t_now->tm_sec)<<"秒"<<std::endl;
		}else{
			std::cerr << " Open the txt file failed ! "<<std::endl;
			return false;
		}
		file_output.close();
		
		std::cout << "topology: 拓扑图文件保存在"<<(file_name)<<std::endl;
		return true;
	}




}
